Package: NetLogoR
Title: Build and Run Spatially Explicit Agent-Based Models
Description: Build and run spatially explicit
    agent-based models using only the R platform. 'NetLogoR' follows the same
    framework as the 'NetLogo' software
    (Wilensky (1999) <https://www.netlogo.org>) and is a translation
    in R of the structure and functions of 'NetLogo'.
    'NetLogoR' provides new R classes to define model agents and functions to
    implement spatially explicit agent-based models in the R environment.
    This package allows benefiting of the fast and easy coding phase from the
    highly developed 'NetLogo' framework, coupled with the versatility, power
    and massive resources of the R software.
    Examples of two models from the NetLogo software repository 
    (Ants <https://ccl.northwestern.edu/netlogo/models/Ants>) and 
    Wolf-Sheep-Predation 
    (<https://ccl.northwestern.edu/netlogo/models/WolfSheepPredation>),
    and a third, Butterfly, from 
    Railsback and Grimm (2012) <https://www.railsback-grimm-abm-book.com/>, all
    written using 'NetLogoR' are available. 
    The 'NetLogo' code of the original version of these
    models is provided alongside.
    A programming guide inspired from the 'NetLogo' Programming Guide
    (<https://docs.netlogo.org/programming.html>) and a dictionary
    of 'NetLogo' primitives (<https://docs.netlogo.org/dictionary.html>)
    equivalences are also available.
    NOTE: To increment 'time', these functions can use a for loop or can be
    integrated with a discrete event simulator, such as 'SpaDES'
    (<https://cran.r-project.org/package=SpaDES>).
URL: https://netlogor.predictiveecology.org,
        https://github.com/PredictiveEcology/NetLogoR/,
        https://groups.google.com/g/netlogor
Version: 1.0.6
Date: 2025-09-02
Authors@R: c(
    person("Sarah", "Bauduin", , "sarahbauduin@hotmail.fr", role = "aut",
           comment = c(ORCID = "0000-0002-3252-5894")),
    person("Eliot J B", "McIntire", , "eliot.mcintire@canada.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6914-8316")),
    person("Alex M", "Chubaty", , "achubaty@for-cast.ca", role = "aut",
           comment = c(ORCID = "0000-0001-7146-8135")),
    person("His Majesty the King in Right of Canada, as represented by the Minister of Natural Resources Canada", role = "cph")
  )
Depends: R (>= 4.3)
Imports: data.table, grDevices, methods, quickPlot (>= 1.0.2), stats,
        terra, utils
Suggests: CircStats, covr, knitr, microbenchmark, raster, rmarkdown,
        sf, sp, SpaDES.core (>= 2.0.3), SpaDES.tools (>= 2.0.5),
        testthat, withr
BugReports: https://github.com/PredictiveEcology/NetLogoR/issues
ByteCompile: yes
Encoding: UTF-8
Language: en-CA
License: GPL-3
RoxygenNote: 7.3.2
VignetteBuilder: knitr, rmarkdown
Collate: 'Agent-classes.R' 'NetLogoR-package.R'
        'worldNLR-classes-methods.R' 'helpers.R'
        'agentMatrix-Class-methods.R' 'agentset-functions.R'
        'function-arguments.R' 'patch-functions.R' 'plot.R'
        'quickPlot.R' 'spades-functions.R' 'turtle-functions.R'
        'world-functions.R'
NeedsCompilation: no
Packaged: 2025-09-02 21:29:33 UTC; achubaty
Author: Sarah Bauduin [aut] (ORCID: <https://orcid.org/0000-0002-3252-5894>),
  Eliot J B McIntire [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6914-8316>),
  Alex M Chubaty [aut] (ORCID: <https://orcid.org/0000-0001-7146-8135>),
  His Majesty the King in Right of Canada, as represented by the Minister
    of Natural Resources Canada [cph]
Maintainer: Eliot J B McIntire <eliot.mcintire@canada.ca>
Repository: CRAN
Date/Publication: 2025-09-02 22:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 14:14:52 UTC; unix
