# Generated by roxygen2: do not edit by hand

S3method(cbind,agentMatrix)
S3method(head,agentMatrix)
S3method(numLayers,worldArray)
S3method(numLayers,worldMatrix)
S3method(plot,agentMatrix)
S3method(plot,worldArray)
S3method(plot,worldMatrix)
S3method(points,agentMatrix)
S3method(rbind,agentMatrix)
S3method(tail,agentMatrix)
export("bbox<-")
export(NLall)
export(NLany)
export(NLcount)
export(NLdist)
export(NLset)
export(NLwith)
export(NLworldIndex)
export(PxcorPycorFromCell)
export(agentMatrix)
export(bk)
export(canMove)
export(cellFromPxcorPycor)
export(clearPatches)
export(createOTurtles)
export(createTurtles)
export(createWorld)
export(die)
export(diffuse)
export(downhill)
export(dx)
export(dy)
export(face)
export(fd)
export(hatch)
export(home)
export(inCone)
export(inRadius)
export(inspect)
export(isNLclass)
export(layoutCircle)
export(left)
export(maxNof)
export(maxOneOf)
export(maxPxcor)
export(maxPycor)
export(minNof)
export(minOneOf)
export(minPxcor)
export(minPycor)
export(moveTo)
export(nOf)
export(neighbors)
export(noPatches)
export(noTurtles)
export(of)
export(oneOf)
export(other)
export(pExist)
export(patch)
export(patchAhead)
export(patchAt)
export(patchDistDir)
export(patchHere)
export(patchLeft)
export(patchRight)
export(patchSet)
export(patches)
export(randomPxcor)
export(randomPycor)
export(randomXYcor)
export(randomXcor)
export(randomYcor)
export(raster2world)
export(right)
export(setXY)
export(sf2turtles)
export(sortOn)
export(spatRast2world)
export(spdf2turtles)
export(sprout)
export(stackWorlds)
export(subHeadings)
export(tExist)
export(towards)
export(turtle)
export(turtleSet)
export(turtles2sf)
export(turtles2spdf)
export(turtlesAt)
export(turtlesOn)
export(turtlesOwn)
export(updateList)
export(uphill)
export(withMax)
export(withMin)
export(world2raster)
export(world2spatRast)
export(worldHeight)
export(worldWidth)
export(wrap)
exportClasses(agentMatrix)
exportClasses(worldArray)
exportClasses(worldMatrix)
exportClasses(worldNLR)
exportMethods("$")
exportMethods("==")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[")
exportMethods("[[<-")
exportMethods(NLall)
exportMethods(NLany)
exportMethods(NLcount)
exportMethods(NLdist)
exportMethods(NLset)
exportMethods(NLwith)
exportMethods(NLworldIndex)
exportMethods(PxcorPycorFromCell)
exportMethods(agentMatrix)
exportMethods(bbox)
exportMethods(bk)
exportMethods(canMove)
exportMethods(cellFromPxcorPycor)
exportMethods(clearPatches)
exportMethods(coordinates)
exportMethods(createOTurtles)
exportMethods(createTurtles)
exportMethods(createWorld)
exportMethods(die)
exportMethods(diffuse)
exportMethods(downhill)
exportMethods(dx)
exportMethods(dy)
exportMethods(extent)
exportMethods(face)
exportMethods(fd)
exportMethods(hatch)
exportMethods(home)
exportMethods(inCone)
exportMethods(inRadius)
exportMethods(initialize)
exportMethods(inspect)
exportMethods(isNLclass)
exportMethods(layerNames)
exportMethods(layoutCircle)
exportMethods(left)
exportMethods(length)
exportMethods(maxNof)
exportMethods(maxOneOf)
exportMethods(maxPxcor)
exportMethods(maxPycor)
exportMethods(minNof)
exportMethods(minOneOf)
exportMethods(minPxcor)
exportMethods(minPycor)
exportMethods(moveTo)
exportMethods(nOf)
exportMethods(neighbors)
exportMethods(nrow)
exportMethods(of)
exportMethods(oneOf)
exportMethods(other)
exportMethods(pExist)
exportMethods(patch)
exportMethods(patchAhead)
exportMethods(patchAt)
exportMethods(patchDistDir)
exportMethods(patchHere)
exportMethods(patchLeft)
exportMethods(patchRight)
exportMethods(patchSet)
exportMethods(patches)
exportMethods(randomPxcor)
exportMethods(randomPycor)
exportMethods(randomXYcor)
exportMethods(randomXcor)
exportMethods(randomYcor)
exportMethods(raster2world)
exportMethods(right)
exportMethods(setXY)
exportMethods(sf2turtles)
exportMethods(show)
exportMethods(sortOn)
exportMethods(spatRast2world)
exportMethods(spdf2turtles)
exportMethods(sprout)
exportMethods(stackWorlds)
exportMethods(subHeadings)
exportMethods(tExist)
exportMethods(towards)
exportMethods(turtle)
exportMethods(turtleSet)
exportMethods(turtles2sf)
exportMethods(turtles2spdf)
exportMethods(turtlesAt)
exportMethods(turtlesOn)
exportMethods(turtlesOwn)
exportMethods(uphill)
exportMethods(withMax)
exportMethods(withMin)
exportMethods(world2raster)
exportMethods(world2spatRast)
exportMethods(worldHeight)
exportMethods(worldWidth)
exportMethods(wrap)
import(methods)
importFrom(data.table,':=')
importFrom(data.table,data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setkey)
importFrom(grDevices,rainbow)
importFrom(graphics,points)
importFrom(quickPlot,.identifyGrobToPlot)
importFrom(quickPlot,layerNames)
importFrom(quickPlot,numLayers)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(terra,ncol)
importFrom(terra,nrow)
importFrom(terra,rast)
importFrom(terra,values)
importFrom(utils,head)
importFrom(utils,tail)
importMethodsFrom(quickPlot,.identifyGrobToPlot)
